"""
User defined code can be added here.
"""
from typing import Union, Tuple
from tmtc_core.pus_tc.tmtcc_pus_tc_base import PusTcInfo
from tmtc_core.tmtc_core_definitions import ComInterfaces
from enum import Enum


# TODO: We really need that JSON file which will not be tracked by version control.
#       In that JSON file, we could set the current developer, so that developers can
#       hook into the config.
class Developer(Enum):
    Robin = 0


Developer = Developer.Robin


def command_preparation_hook() -> Tuple[bytearray, Union[None, PusTcInfo]]:
    """
    Can be used to pack user-defined commands.
    """
    if Developer == Developer.Robin:
        return prepare_robins_commands()


def prepare_robins_commands():
    from tmtc_core.pus_tc.tmtcc_pus_tc_base import PusTelecommand
    command = PusTelecommand(service=17, subservice=1, ssc=20)
    return command.pack_command_tuple()

def global_setup_hook():
    """
    Can be used to alter the global variables in a custom defined way.
    For example, device specific com ports or ethernet ports can be set here.
    The global variables in the config.obsw_config file can be edited here
    by using the handle.
    """
    if Developer == Developer.Robin:
        global_setup_hook_robin()


def global_setup_hook_robin():
    from config.tmtcc_config import get_glob_com_if, set_glob_apid
    if get_glob_com_if() == ComInterfaces.Ethernet:
        # Configure APID for FSFW example. Set this back to 0x73 for STM32!
        set_glob_apid(0xEF)
