#!/usr/bin/python3
"""
@brief      Core method called by entry point files to initiate the TMTC commander
@details
@manual
@author     R. Mueller
"""
import sys

from config.tmtcc_version import SW_VERSION, SW_SUBVERSION
from tmtc_core.utility.obsw_logger import set_tmtc_logger, get_logger

from config.tmtcc_config import set_globals
import config.tmtcc_config as g
from core.tmtc_backend import TmTcHandler
from core.tmtc_frontend import TmTcFrontend
from utility.tmtcc_args_parser import parse_input_arguments

LOGGER = get_logger()


def run_tmtc_client(use_gui: bool, reduced_printout: bool = False):
    """
    Main method, reads input arguments, sets global variables and start TMTC handler.
    """
    if not reduced_printout:
        print("-- Python TMTC Commander --")
        if use_gui:
            print("-- GUI mode --")
        else:
            print("-- Command line mode --")
        print("-- Software version  v" + str(SW_VERSION) + "." + str(SW_SUBVERSION) + " --")
    set_tmtc_logger()
    LOGGER.info("Starting TMTC Client..")

    if not use_gui:
        LOGGER.info("Parsing input arguments..")
        args = parse_input_arguments()

        LOGGER.info("Setting global variables..")
        set_globals(args)

    LOGGER.info("Starting TMTC Handler..")

    if not use_gui:
        # The global variables are set by the argument parser.
        tmtc_handler = TmTcHandler(g.get_glob_com_if(), g.get_glob_mode())
        tmtc_handler.set_one_shot_or_loop_handling(g.G_LISTENER_AFTER_OP)
        tmtc_handler.initialize()
        tmtc_handler.start()
    else:
        from PyQt5.QtWidgets import QApplication
        app = QApplication(["TMTC Commander"])
        tmtc_gui = TmTcFrontend()
        tmtc_gui.start_ui()
        sys.exit(app.exec_())




